var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// index.ts
var foundation_exports = {};
__export(foundation_exports, {
  EventBus: () => EventBus,
  Handler: () => handler_exports,
  Hooks: () => Hooks,
  UserActionLock: () => instance,
  Utils: () => utils_exports
});
module.exports = __toCommonJS(foundation_exports);

// handler/index.ts
var handler_exports = {};
__export(handler_exports, {
  createStaticHandler: () => createStaticHandler,
  useExposeHandler: () => useExposeHandler,
  useHandler: () => useHandler
});
var import_react = require("react");
var createStaticHandler = () => {
  return {};
};
var useHandler = () => {
  const _handler = (0, import_react.useRef)({});
  return _handler.current;
};
var useExposeHandler = (handler, methods, dependency = []) => {
  (0, import_react.useEffect)(() => {
    if (handler) {
      Object.assign(handler, methods);
    }
    return () => {
      if (handler) {
        Object.assign(handler, {});
      }
    };
  }, dependency);
};

// event-bus/index.ts
var import_eventemitter3 = __toESM(require("eventemitter3"));
var eventEmitter = new import_eventemitter3.default();
var listen = (eventName, callback) => {
  eventEmitter.on(eventName, callback);
};
var listenOnce = (eventName, callback) => {
  eventEmitter.once(eventName, callback);
};
var remove = (eventName, callback) => {
  eventEmitter.removeListener(eventName, callback);
};
var emit = (eventName, payload) => {
  eventEmitter.emit(eventName, payload);
};
var EventBus = {
  listen,
  listenOnce,
  remove,
  emit
};

// utils/index.ts
var utils_exports = {};
__export(utils_exports, {
  Stack: () => Stack,
  randomString: () => randomString
});

// utils/random-string.ts
var numbers = "0123456789";
var letters = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
var specials = "!$%^&*()_+|~-=`{}[]:;<>?,./";
function _defaults(opts) {
  return {
    length: opts.length || 8,
    numeric: typeof opts.numeric === "boolean" ? opts.numeric : true,
    letters: typeof opts.letters === "boolean" ? opts.letters : true,
    special: typeof opts.special === "boolean" ? opts.special : false
  };
}
function _buildChars(opts) {
  var chars = "";
  if (opts.numeric) {
    chars += numbers;
  }
  if (opts.letters) {
    chars += letters;
  }
  if (opts.special) {
    chars += specials;
  }
  return chars;
}
var randomString = (_opts) => {
  let opts = _defaults(_opts);
  var i, rn, rnd = "", len = opts.length, randomChars = _buildChars(opts);
  for (i = 1; i <= len; i++) {
    rnd += randomChars.substring(
      rn = Math.floor(Math.random() * randomChars.length),
      rn + 1
    );
  }
  return rnd;
};

// utils/stack.ts
var Stack = class {
  constructor() {
    this.items = [];
    this.count = 0;
  }
  getLength() {
    return this.count;
  }
  push(item) {
    this.items.push(item);
    this.count = this.count + 1;
  }
  pop() {
    if (this.count > 0) {
      this.count = this.count - 1;
    }
    return this.items.pop();
  }
  peek() {
    return this.items.slice(-1)[0];
  }
  nearLast() {
    if (this.items.length > 1) {
      return this.items[this.items.length - 2];
    } else {
      return this.peek();
    }
  }
};

// user-action-lock.ts
var UserActionLock = class {
  constructor() {
    this.keys = /* @__PURE__ */ new Set();
  }
  lock(key) {
    if (this.keys.has(key)) {
      return false;
    }
    this.keys.add(key);
    return true;
  }
  lockAutoRelease({
    key,
    afterDurationMs
  }) {
    if (this.keys.has(key)) {
      return false;
    }
    this.keys.add(key);
    this.autoRelease({ key, afterDurationMs });
    return true;
  }
  release(key) {
    this.keys.delete(key);
  }
  autoRelease({ key, afterDurationMs }) {
    setTimeout(() => {
      this.keys.delete(key);
    }, afterDurationMs);
  }
  create() {
    return new UserActionLock();
  }
};
var instance = new UserActionLock();

// use-user-action-lock.ts
var use_user_action_lock_exports = {};
__export(use_user_action_lock_exports, {
  useUserActionLock: () => useUserActionLock
});
var import_react2 = require("react");
var useUserActionLock = (delay = 1e3) => {
  const lockKey = (0, import_react2.useRef)(randomString({ length: 16 }));
  const runAction = (func) => {
    if (instance.lockAutoRelease({
      key: lockKey.current,
      afterDurationMs: delay
    })) {
      func();
    }
  };
  return runAction;
};

// use-runner.ts
var use_runner_exports = {};
__export(use_runner_exports, {
  useRunner: () => useRunner
});
var import_react3 = require("react");
var useRunner = () => {
  const actionLock = (0, import_react3.useRef)(instance.create()).current;
  const runOnce = (config) => {
    if (actionLock.lock(config.lockKey)) {
      config.func();
    }
  };
  (0, import_react3.useEffect)(() => {
  }, []);
  return { runOnce };
};

// use-delay-event-handler.ts
var use_delay_event_handler_exports = {};
__export(use_delay_event_handler_exports, {
  useDelayEventHandler: () => useDelayEventHandler,
  useDelayEventHandlerEx: () => useDelayEventHandlerEx
});
var import_react4 = require("react");
var useDelayEventHandler = (event, { delayUntilRepeat }) => {
  const shoudleFireEvent = (0, import_react4.useRef)(true);
  const fireEvent = (argv) => {
    if (delayUntilRepeat === -1) {
      event(argv);
      return;
    }
    if (shoudleFireEvent.current) {
      shoudleFireEvent.current = false;
      event(argv);
      setTimeout(() => {
        shoudleFireEvent.current = true;
      }, delayUntilRepeat);
    }
  };
  return fireEvent;
};
var useDelayEventHandlerEx = ({
  delayUntilRepeat
}) => {
  const shoudleFireEvents = (0, import_react4.useRef)(
    /* @__PURE__ */ new Map()
  ).current;
  const createEventKeyIfAny = (eventKey) => {
    if (!shoudleFireEvents.has(eventKey)) {
      shoudleFireEvents.set(eventKey, true);
    }
  };
  const fireEvent = (eventConfig) => {
    createEventKeyIfAny(eventConfig.eventKey);
    const _delayUntilRepeat = eventConfig.delayUntilRepeat || delayUntilRepeat;
    if (_delayUntilRepeat === -1) {
      eventConfig.event();
      return;
    }
    const shoudleFireEvent = shoudleFireEvents.get(eventConfig.eventKey);
    if (shoudleFireEvent) {
      shoudleFireEvents.set(eventConfig.eventKey, false);
      eventConfig.event();
      setTimeout(() => {
        shoudleFireEvents.set(eventConfig.eventKey, true);
      }, _delayUntilRepeat);
    }
  };
  return fireEvent;
};

// index.ts
var Hooks = {
  ...use_delay_event_handler_exports,
  ...use_runner_exports,
  ...use_user_action_lock_exports
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  EventBus,
  Handler,
  Hooks,
  UserActionLock,
  Utils
});
